package de.fzj.unicore.bes.util;

import java.io.File;

import javax.xml.namespace.QName;

import org.apache.xmlbeans.XmlCursor;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;
import org.w3.x2005.x08.addressing.ReferenceParametersType;
import org.xmlsoap.schemas.soap.envelope.Fault;

import de.fzj.unicore.wsrflite.utils.WSServerUtilities;

public class BESUtils {

	public static Fault buildSOAP11Fault(QName faultCode, String faultString) {
		Fault soapFault = Fault.Factory.newInstance();
		// when policies will be applied then appropriate error message will be
		// generated
		soapFault.setFaultcode(faultCode);
		soapFault.setFaultstring(faultString);
		return soapFault;
	}

	public static EndpointReferenceType makeSimpleEPR(String serviceUrl) {
		EndpointReferenceType eprt = WSServerUtilities.newEPR();
		eprt.addNewAddress().setStringValue(serviceUrl);
		return eprt;
	}
	
	public static EndpointReferenceType makeSimpleEPR(String serviceUrl, String resourceId) {
		EndpointReferenceType eprt = WSServerUtilities.newEPR();
		eprt.addNewAddress().setStringValue(serviceUrl + "?res=" + resourceId);
		WSServerUtilities.addUGSRefparamToEpr(eprt,resourceId);
		return eprt;
	}
	
	/**
	 * This method extracts the incoming epr's resourceid within the reference parameters. 
	 *  
	 * @return referenceParam 
	 * */
	public static String extractRefParamResourceId(EndpointReferenceType eprt) {
		ReferenceParametersType refParam = eprt.getReferenceParameters();
		if(refParam == null) return null;
		XmlCursor cursor = refParam.newCursor();
		cursor.toFirstChild();
		String rid =  cursor.getTextValue();
		cursor.dispose();
		return rid;
	}
	
	public static JobDefinitionDocument getJSDLFromFile(String filePath) throws Exception{
			JobDefinitionDocument jobDoc = null;
			jobDoc = JobDefinitionDocument.Factory.parse(new File(filePath));
			return jobDoc;
			
	}

	public static void deleteFile(File f){
		if (!f.isDirectory()) {
			f.delete();
		} else {
			for (File child : f.listFiles()) {
				deleteFile(child);
			}
		}
	}
}



