package de.fzj.unicore.uas.client;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.util.HashMap;
import java.util.Map;

import org.junit.Test;
import org.unigrids.services.atomic.types.MetadataType;
import org.unigrids.services.atomic.types.TextInfoType;

public class TestMetadataClient {
	
	@Test
	public void testConvertMetadataToMap(){
		MetadataType meta=MetadataType.Factory.newInstance();
		meta.setContentMD5("ABCD");
		meta.setContentType("text/plain");
		meta.setTagArray(new String[]{"foo","bar"});
		TextInfoType p=meta.addNewProperty();
		p.setName("x");
		p.setValue("value-of-x");
		Map<String, String> map=MetadataClient.asMap(meta);
		assertNotNull(map);
		assertEquals("ABCD",map.get(MetadataClient.CONTENT_MD5));
		assertEquals("text/plain",map.get(MetadataClient.CONTENT_TYPE));
		assertEquals("foo,bar",map.get(MetadataClient.TAGS));
		assertEquals("value-of-x",map.get("x"));
	}
	
	@Test
	public void testConvertMapToMeta(){
		Map<String, String> map=new HashMap<String, String>();
		map.put("x", "value-of-x");
		map.put("Content-Type", "text/plain");
		map.put("Content-MD5", "ABCD");
		map.put("Tags", "foo,bar");
		MetadataType md=MetadataClient.convert(map);
		assertEquals(2, md.getTagArray().length);
		assertEquals(1, md.getPropertyArray().length);
		assertEquals("ABCD",md.getContentMD5());
		assertEquals("text/plain",md.getContentType());
	}
}
