package de.fzj.unicore.uas.impl;

import java.io.StringWriter;
import java.security.cert.X509Certificate;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlObject;
import org.bouncycastle.openssl.PEMWriter;
import org.unigrids.services.atomic.types.AcceptedVOsType;
import org.unigrids.services.atomic.types.SecurityDocument;

import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.wsrflite.security.SecurityManager;
import de.fzj.unicore.wsrflite.xmlbeans.ResourceProperty;
import de.fzj.unicore.wsrflite.xmlbeans.impl.WSResourceImpl;

/**
 * publishes server security info (CAs, trusted VOs)
 *
 * @author schuller
 * @author golbi
 */
public class SecurityInfoResourceProperty extends ResourceProperty<Object>{

	private static final Logger logger=LogUtil.getLogger(LogUtil.SERVICES,SecurityInfoResourceProperty.class);

	private static final long serialVersionUID = 1L;

	private SecurityDocument[] xdoc;

	private final boolean addServerCert;

	/**
	 * create a new security info property
	 * @param parent - the parent resource
	 * @param addServerCert - whether to publish the server cert
	 */
	public SecurityInfoResourceProperty(WSResourceImpl parent, boolean addServerCert){
		super(parent);
		this.addServerCert=addServerCert;
		this.xdoc=new SecurityDocument[1];
		xdoc[0]=SecurityDocument.Factory.newInstance();
		xdoc[0].addNewSecurity();
		doGetInfo();
	}

	@Override
	public XmlObject[] getXml() {
		return xdoc;
	}

	protected void doGetInfo(){
		//server cert in pem format
		if(addServerCert){
			try{
				X509Certificate c=SecurityManager.getServerCert();
				if(c!=null){
					StringWriter writer=new StringWriter();
					PEMWriter pw=new PEMWriter(writer);
					pw.writeObject(c);
					pw.flush();
					xdoc[0].getSecurity().setServerCertificate(writer.toString());
				}
			}catch(Exception ex){
				LogUtil.logException("Error getting server cert", ex, logger);
			}
		}
		//Trusted CAs
		String[]trusted=SecurityManager.getTrustedCertificateDNs();
		if(trusted!=null && trusted.length>0){
			xdoc[0].getSecurity().setTrustedCAArray(trusted);
		}
		//VOs
		try{
			String[] vos=SecurityManager.getAtributeSource().getAcceptedVOs();
			if(vos!=null && vos.length>0){
				AcceptedVOsType acceptedVOs=AcceptedVOsType.Factory.newInstance();
				acceptedVOs.setVOArray(vos);
				xdoc[0].getSecurity().setAcceptedVOs(acceptedVOs);
			}
		}
		catch(Exception ex){
			LogUtil.logException("Error getting vo list", ex, logger);
		}
	}

	public SecurityInfoResourceProperty update(){
		doGetInfo();
		return this;
	}

}
