package de.fzj.unicore.uas.impl.sms;

import java.util.Map;

import org.apache.log4j.Logger;

import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.uas.xnjs.XNJSFacade;
import de.fzj.unicore.xnjs.ems.ExecutionException;

/**
 * Represents a HOME storage. The storage root is the current user's 
 * home directory on the target system.
 * @author schuller
 *
 */
public class HomeStorageImpl extends SMSBaseImpl {

	private static final Logger logger=LogUtil.getLogger(LogUtil.SERVICES,HomeStorageImpl.class);

	private transient String homeDirectory=null;

	public HomeStorageImpl() {
		super();
	}

	@Override
	public void initialise(String name, Map<String,Object>initobjs)throws Exception{
		super.initialise(name, initobjs);
		//important - detect problems with establishing HOME dir from XNJS/TSI early.
		setHome();
	}

	private void setHome() throws ExecutionException {
		homeDirectory=XNJSFacade.get(xnjsReference).getGrounder().
		incarnatePath("","HOME",null,getClient());
		if(logger.isDebugEnabled()){
			String xlogin="n/a";
			String dn = "unknown";
			if(getClient() != null && getClient().getXlogin().getUserName() != null){
				xlogin=getClient().getXlogin().getUserName();
			}
			if (getClient() != null && getClient().getDistinguishedName() != null)
				dn = getClient().getDistinguishedName();
			logger.debug("Got home directory for "+
					dn
					+" xlogin='"+xlogin+"'"
					+": <"+homeDirectory+">");
		}
		if (!homeDirectory.endsWith(getSeparator())) 
			homeDirectory = homeDirectory + getSeparator();
	}

	@Override
	protected String getStorageRoot()throws ExecutionException{
		if (homeDirectory == null) {
			try {
				setHome();
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}
		return homeDirectory;
	}

	@Override
	protected void smsPostActivate() {
		homeDirectory=null;
	}



}
