package de.fzj.unicore.uas.impl.tss;

import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

import de.fzj.unicore.persist.PersistenceException;
import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.impl.job.JobManagementImpl;
import de.fzj.unicore.uas.util.LogUtil;
import de.fzj.unicore.uas.xnjs.XNJSFacade;
import de.fzj.unicore.wsrflite.Home;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.xnjs.ems.Action;
import eu.unicore.security.Client;

/**
 * re-create JMS instances from XNJS jobs
 * 
 * @author schuller
 */
public class GenerateJMSInstances implements Runnable{

	private final Logger logger=LogUtil.getLogger(LogUtil.SERVICES, GenerateJMSInstances.class);
	
	private final String tssID;
	
	private final Client client;
	
	private final Map<String,Object>securityContext;
	
	private final String xnjsReference;
	
	private final Home tssHome;
	
	private final Home jms;
	
	private final Set<String>existingJobs;
	
	public GenerateJMSInstances(String tssID, Client client, Map<String,Object>securityContext, String xnjsReference)throws PersistenceException{
		this.tssID=tssID;
		this.client=client;
		this.xnjsReference=xnjsReference;
		this.tssHome=Kernel.getKernel().getHome(UAS.TSS);
		this.jms=Kernel.getKernel().getHome(UAS.JMS);
		this.securityContext=securityContext;
		this.existingJobs=getExistingJobs();
	}
	
	public void run(){
		try{
		logger.info("Regenerating UNICORE jobs for "+client.getDistinguishedName());
		XNJSFacade xnjs=XNJSFacade.get(xnjsReference);
		List<String> jobIDs=xnjs.listJobIDs(client);
		for(String jobID: jobIDs){
			Action action=xnjs.getAction(jobID);
			if(accept(action) && !jobExists(jobID)){
				try{
					logger.info("Restoring "+jobID+" for "+client.getDistinguishedName());
					add(action);
				}catch(Exception ex){
					logger.error("Could not restore job for : "+action.getUUID(),ex);
				}
			}
		}
		}catch(Exception ex){
			logger.error("Could not restore jobs for "+client.getDistinguishedName());
		}
	}
	
	protected boolean accept(Action action){
		//only accept jsdl actions
		String type=action.getType();
		if(!"JSDL".equals(type))return false;
		//and those which belong to the current user
		String owner=action.getClient().getDistinguishedName();
		return client.getDistinguishedName().equalsIgnoreCase(owner);
	}
	
	protected boolean jobExists(String uid){
		return existingJobs.contains(uid);
	}
	
	protected void add(Action action)throws Exception{
		TargetSystemImpl tss=(TargetSystemImpl)tssHome.getForUpdate(tssID);
		try{
			Map<String,Object> map=new HashMap<String,Object>();
			map.put(TargetSystemImpl.INITPARAM_SECURITYCONTEXT, securityContext);
			map.put(TargetSystemImpl.INITPARAM_CLIENT,client);
			map.put(JobManagementImpl.INITPARAM_ACTION,action);
			map.put(JobManagementImpl.INITPARAM_ACTION_NO_XNJS_SUBMIT,Boolean.TRUE);
			map.put(JobManagementImpl.INITPARAM_TSS_UNIQUEID, tssID);
			map.put(TargetSystemImpl.INITPARAM_XNJS_REFERENCE, xnjsReference);
			tss.createJob(map);
		}finally{
			if(tss!=null){
				tssHome.persist(tss);
			}
		}
	}
	
	private Set<String>getExistingJobs()throws PersistenceException{
		return new HashSet<String>(Arrays.asList(jms.getStore().getUniqueIDs()));
	}
	
}
