package de.fzj.unicore.uas.functional;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.client.TaskClient;
import de.fzj.unicore.uas.impl.task.TaskImpl;
import de.fzj.unicore.wsrflite.Home;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.utils.WSServerUtilities;
import eu.unicore.bugsreporter.annotation.FunctionalTest;


public class TestAddressUpdate extends TestCase {

	@FunctionalTest(id="testAddressUpdate",
		description="Tests that published EPRs change when the server URL is re-configured")
	public void testAddressUpdate()throws Exception{
		UAS uas=new UAS("src/test/resources/minimal/wsrflite.xml");
		uas.startSynchronous();
		
		Home taskHome=Kernel.getKernel().getHome(UAS.TASK);
		assertNotNull(taskHome);
		String baseURL=Kernel.getKernel().getProperty(Kernel.WSRF_BASEURL);
		
		EndpointReferenceType parent=EndpointReferenceType.Factory.newInstance();
		parent.addNewAddress().setStringValue(baseURL+"/foo?res=bar");
		Map<String,Object>initParams=new HashMap<String, Object>();
		initParams.put(TaskImpl.INIT_PARENT_SERVICE, parent);
		String taskID=taskHome.createWSRFServiceInstance(initParams);
		EndpointReferenceType taskEPR=WSServerUtilities.makeEPR(UAS.TASK, taskID);
		System.out.println(taskEPR);
		TaskClient client=new TaskClient(taskEPR,Kernel.getKernel().getSecurityProperties());
		client.setUpdateInterval(-1);
		EndpointReferenceType submissionRef=client.getResourcePropertiesDocument().getTaskProperties().getSubmissionServiceReference();
		assertNotNull(submissionRef);
		String url=submissionRef.getAddress().getStringValue();
		assertTrue(url.toString().contains("localhost"));
		
		//OK, now change the base URL... and check it is actually changed
		Kernel.getKernel().setProperty(Kernel.WSRF_BASEURL, "http://127.0.0.1:65321/services");
		submissionRef=client.getResourcePropertiesDocument().getTaskProperties().getSubmissionServiceReference();
		assertNotNull(submissionRef);
		url=submissionRef.getAddress().getStringValue();
		assertTrue(url.toString().contains("127.0.0.1"));
		
	}
}
