package de.fzj.unicore.uas.testsuite;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.util.List;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionType;
import org.junit.Test;
import org.unigrids.x2006.x04.services.tss.SubmitDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.Base;
import de.fzj.unicore.uas.TargetSystemFactory;
import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.client.TSFClient;
import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.xmlbeans.client.RegistryClient;
import eu.unicore.bugsreporter.annotation.FunctionalTest;

public class TestTSSReinit extends Base {
	String url;
	EndpointReferenceType epr;

	@FunctionalTest(id="TSSReinit", 
			description="Tests TargetSystemService behaviour when re-created by a user")
	@Test
	public void testRecreate()throws Exception{
		url=Kernel.getKernel().getProperty(Kernel.WSRF_BASEURL);
		epr=EndpointReferenceType.Factory.newInstance();
		epr.addNewAddress().setStringValue(url+"/"+UAS.REG+"?res=default_registry");
		RegistryClient reg=new RegistryClient(epr,Kernel.getKernel().getSecurityProperties());
		//find a TargetSystemFactory
		List<EndpointReferenceType> tsfs=reg.listServices(TargetSystemFactory.TSF_PORT);
		EndpointReferenceType tsfepr=findFirstAccessibleService(tsfs);
		TSFClient tsf=new TSFClient(tsfepr,Kernel.getKernel().getSecurityProperties());
		tsf.setUpdateInterval(-1);
		//clean up all TSSs to avoid dependencies on other tests...
		for(EndpointReferenceType tEpr: tsf.getTargetSystems()){
			new TSSClient(tEpr,Kernel.getKernel().getSecurityProperties()).destroy();
		}
		assertTrue(tsf.getTargetSystems().size()==0);
		TSSClient tss = tsf.createTSS();
		assertTrue(tsf.getTargetSystems().size()==1);
		
		runJob(tss);
		runJob(tss);
		runJob(tss);
		Thread.sleep(5000);
		tss.setUpdateInterval(-1);
		long nj=tss.getJobs().size();
		System.out.println("Have <"+nj+"> jobs");
		
		tss.destroy();
		
		tss=tsf.createTSS();
		//need to wait a bit...
		Thread.sleep(2000);
		assertEquals(nj, tss.getJobs().size());

	}

	
	private void runJob(TSSClient tss)throws Exception{
		SubmitDocument sd=SubmitDocument.Factory.newInstance();
		sd.addNewSubmit().setAutoStartWhenReady(true);
		JobDefinitionType jobDefinition=JobDefinitionType.Factory.newInstance();
		jobDefinition.addNewJobDescription().addNewApplication().setApplicationName("Date");
		sd.getSubmit().setJobDefinition(jobDefinition);
		tss.submit(sd);
	}


}
