package eu.unicore.genii.security;

import java.io.ByteArrayInputStream;
import java.io.OutputStream;
import java.math.BigInteger;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.cert.X509Certificate;
import java.util.Date;
import java.util.Random;

import javax.security.auth.x500.X500Principal;

import org.bouncycastle.x509.X509V3CertificateGenerator;

import de.fzj.unicore.xnjs.Configuration;
import de.fzj.unicore.xnjs.ems.Action;
import de.fzj.unicore.xnjs.tsi.TSI;

import eu.unicore.genii.GenesisConstants;
import eu.unicore.security.Client;
import eu.unicore.security.util.client.IClientProperties;

public class TrustDelegationUtils implements GenesisConstants {
	
	public static GenesisClientProperties getGenesisClientProps(Client client, IClientProperties unicoreXSecurityProperties) throws Exception
	{
		long startTime = System.currentTimeMillis();
		long endTime = startTime + 30*3600*1000;
		int keyLength = 512;
		String signatureAlgorithm="SHA1withRSA";
		
		X509V3CertificateGenerator certGen = new X509V3CertificateGenerator();
		KeyPairGenerator kpg=KeyPairGenerator.getInstance(unicoreXSecurityProperties.getPrivateKey().getAlgorithm());
		kpg.initialize(keyLength);
		KeyPair pair = kpg.generateKeyPair();
		PrivateKey privateKey = pair.getPrivate();
		
		Random rand=new Random();
		X500Principal subjectDN = new X500Principal("CN="+Integer.toString(rand.nextInt(9999))+"-unprivilegedcert");
		
		certGen.setSerialNumber(new BigInteger(20,rand));
		certGen.setIssuerDN(unicoreXSecurityProperties.getCertificateChain()[0].getSubjectX500Principal());
		certGen.setSubjectDN(subjectDN);
		certGen.setNotBefore(new Date(startTime));
		certGen.setNotAfter(new Date(endTime));
		certGen.setPublicKey(pair.getPublic());
		certGen.setSignatureAlgorithm(signatureAlgorithm);
		
		X509Certificate certificate = certGen.generate(unicoreXSecurityProperties.getPrivateKey());
		
		certificate.checkValidity(new Date());
		certificate.verify(unicoreXSecurityProperties.getPublicKey().getPublicKey());
		GenesisClientProperties result = new GenesisClientProperties(certificate,privateKey);
		return result;
	}
	

}
