package de.fzj.unicore.uas.fts.udt;

import java.io.File;

import junit.framework.TestCase;

import org.unigrids.services.atomic.types.ProtocolType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.client.StorageFactoryClient;
import de.fzj.unicore.uas.client.UDTClient;
import de.fzj.unicore.wsrflite.Kernel;

public class TestUDT extends TestCase {

	protected String getConfigPath(){
		return "src/test/resources/uas.config";
	}

	private void deleteFile(File f){
		if(!f.exists())return;

		if(!f.isDirectory()){
			f.delete();
		}
		else{
			for(File child: f.listFiles()){
				deleteFile(child);
			}
		}
	}

	static boolean haveInit=false;

	protected void setUp() throws Exception{
		if(haveInit)return;
		haveInit=true;
		long start=System.currentTimeMillis();

		//clear data directory
		deleteFile(new File("target","data"));

		File f=new File("target/data/teststorage");
		if(!f.exists())f.mkdirs();

		UAS uas=new UAS(getConfigPath());
		uas.startSynchronous();

		System.out.println("Startup time: "+(System.currentTimeMillis()-start)+" ms.");
	}

	public void test1()throws Exception{
		EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
		epr.addNewAddress().
		setStringValue("http://localhost:65321/services/StorageFactory?res=default_storage_factory");
		StorageFactoryClient smf=new StorageFactoryClient(epr,Kernel.getKernel().getSecurityProperties());
		StorageClient sms=smf.createSMS();
		ProtocolType.Enum[]ps=sms.getSupportedProtocols();
		boolean foundUDT=false;
		for(ProtocolType.Enum p: ps){
			System.out.println("Supported: "+p);
			if(ProtocolType.UDT.equals(p))foundUDT=true;
		}
		assertTrue(foundUDT);
		UDTClient ftc=(UDTClient)sms.getImport("test", ProtocolType.UDT);
		assertNotNull(ftc);
	}

}

